/*
	libprop -- library to simulate and measure radio signal propagation
	
	Copyright (C) 2007 Jeffrey Sharkey, jsharkey.org
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "source.h"
#include "geom.h"
#include "radio.h"
#include "utils.h"
#include "testcases.h"


#include <stdio.h>
#include <stdlib.h>

#include "gd.h"

#include <iostream>
#include <fstream>

using namespace std;



void error() {
	cout << "Content-Type:image/png" << endl;
	cout << "Error: true" << endl << endl;
	gdImagePtr im = gdImageCreateTrueColor(10, 10);
	gdImageFilledRectangle(im, 0, 0, 10, 10, gdTrueColor(0, 0, 0));
	gdImagePng(im, stdout);
	exit(0);

}


int main(void) {

//cout << "Content-Type:text/plain" << endl << endl;

double towerLat = -1, towerLon = -1, topRightLat = -1, topRightLon = -1, bottomLeftLat = -1, bottomLeftLon = -1;
int detail = -1, r = -1, g = -1, b = -1;
double txHeight = -1, rxHeight = -1, eirp = -1, mhz = -1, txantenna = -1, rxantenna = -1, sens = -1;

// read in all variables from query
char* vars = getenv("QUERY_STRING");
char* var = strtok(vars, ",");
while(var != NULL) {
	if(towerLat == -1) towerLat = atof(var);
	else if(towerLon == -1) towerLon = atof(var);
	else if(bottomLeftLat == -1) bottomLeftLat = atof(var);
	else if(bottomLeftLon == -1) bottomLeftLon = atof(var);
	else if(topRightLat == -1) topRightLat = atof(var);
	else if(topRightLon == -1) topRightLon = atof(var);
	else if(detail == -1) detail = atoi(var);
	else if(r == -1) r = atoi(var);
	else if(g == -1) g = atoi(var);
	else if(b == -1) b = atoi(var);
	else if(txHeight == -1) txHeight = atof(var);
	else if(rxHeight == -1) rxHeight = atof(var);
	else if(eirp == -1) eirp = atof(var);
	else if(mhz == -1) mhz = atof(var);
	else if(txantenna == -1) txantenna = atof(var);
	else if(rxantenna == -1) rxantenna = atof(var);
	else if(sens == -1) sens = atof(var);
	var = strtok(NULL, ",");
}

// check for problems with incoming data bounds

if(topRightLat < 33 || topRightLat > 49 ||
	bottomLeftLat < 33 || bottomLeftLat > 49 ||
	towerLat < 33 || towerLat > 49) error();

if(topRightLon > -102 || topRightLon < -126 ||
	bottomLeftLon > -102 || bottomLeftLat < -126 ||
	towerLon > -102 || towerLon < -126) error();

if(detail < 50) error();

if(r < 0 || r > 255 ||
	g < 0 || g > 255 ||
	b < 0 || b > 255) error();

if(txHeight < 0 || txHeight > 100 ||
	rxHeight < 0 || rxHeight > 100) error();

if(eirp < 0 || eirp > 32768) error();
if(mhz < 150 || mhz > 10000) error();

if(txantenna < 0 || txantenna > 30 ||
	rxantenna < 0 || rxantenna > 30) error();

// load regional elevation data
SourceGroup* sg = new SourceGroup();
//sg->add(new SourceGridFloat(new Convert(), TYPE_ELEV, "80214271.elev/80214271.hdr", false));
//sg->add(new SourceGridFloat(new Convert(), TYPE_ELEV, "63034346/63034346.hdr", false));


Convert* normal = new Convert();

//sg->add(new SourceGridFloat(normal, TYPE_ELEV, "63034346/63034346.hdr", 2809, 5336, -108.34009259127, 43.249999999636, 0.000092592592600001));


sg->add(new SourceGridFloat(normal, TYPE_ELEV, "wyoming/58366835/58366835.hdr", 5401, 5401, -108.50009259129, 42.999999999616, 0.000092592592600001));
sg->add(new SourceGridFloat(normal, TYPE_ELEV, "wyoming/89313062/89313062.hdr", 5401, 5400, -108.50009259129, 43.499999999656, 0.000092592592600001));



// create image for writing

// one meter in degrees notation
double mlat = 0.0000090175;
double mlon = 0.0000120425;

// pixels is 
int width = (int)((topRightLon - bottomLeftLon) / (mlon * detail));
int height = (int)((topRightLat - bottomLeftLat) / (mlat * detail));

if(width > 50 || height > 50) error();


gdImagePtr im = gdImageCreateTrueColor(width, height);
gdImageAlphaBlending(im, 0);
gdImageSaveAlpha(im, 1);

// find central tower location
//Point* tower = new Point((bottomLeftLat + topRightLat) / 2, (bottomLeftLon + topRightLon) / 2, txHeight);
Point* tower = new Point(towerLat, towerLon, txHeight);

// draw across entire image
double degResLat = (topRightLat - bottomLeftLat) / height;
double degResLon = (topRightLon - bottomLeftLon) / width;
int y = height - 1;
for(double lat = bottomLeftLat; lat < topRightLat; lat += degResLat) {
	int x = 0;
	for(double lon = bottomLeftLon; lon < topRightLon; lon += degResLon) {
		// create and test point
		Point* test = new Point(lat, lon, rxHeight);

double loss;
if(sg->contains(test)) {
	loss = pathLoss(tower, test, sg, 0.030, eirp, txantenna+rxantenna, mhz);
} else {
	loss = -1024;
}

		// color if coverage exists
		int color;
		if(loss < sens || loss == DENIED) {
			color = gdTrueColorAlpha(0, 0, 0, 127);
		} else {
			// scale red between 0 and -100
			if(loss > 0) loss = 0;
			double val = (64 * (loss/sens));
			color = gdTrueColorAlpha(r, g, b, (int)val);
		}

		gdImageSetPixel(im, x, y, color);
		x++;
	}
	y--;
}

cout << "Content-Type:image/png" << endl;
cout << "Error: false" << endl;
cout << "Size: " << width << "x" << height << endl;
cout << endl;
gdImagePng(im, stdout);

return 0;

}

